﻿/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System.Collections.ObjectModel;
using Xamarin.Forms;

namespace StopWatch
{
    /// <summary>
    /// StopWatch app's main page, UI part
    /// </summary>
    public partial class MainPage : ContentPage
    {
        private double layoutTopHeight;
        private double layoutWidth;
        private Label timeLabel;
        private Label minorLabel;
        private Label scaleLabel;
        private Image startButton;
        private Image labButton;
        private ListView lapListView;
        private ObservableCollection<LapListItem> lapList;

        /// <summary>
        /// Create UI components
        /// </summary>
        /// <param name="screenWidth">The Screen's Width</param>
        /// <param name="screenHeight">The Screen's Height</param>
        void InitializeComponent(int screenWidth, int screenHeight)
        {
            layoutTopHeight = 0.46 * screenHeight;
            layoutWidth = screenWidth;
            Title = "STOPWATCH";

            // UI Logic Development
            // Layout UI components by using AbsoluteLayout.
            timeLabel = new Label()
            {
                Text = "00:00:00",
                TextColor = new Color(255, 255, 255),
                HorizontalTextAlignment = TextAlignment.End,
                VerticalOptions = LayoutOptions.End,
                FontSize = 60,
                IsVisible = true,
            };
            // Init the minorLabel
            minorLabel = new Label()
            {
                Text = ".00",
                TextColor = new Color(255, 255, 255),
                VerticalOptions = LayoutOptions.End,
                FontSize = 40,
                IsVisible = true,
            };
            //Init the timeLineLayout
            StackLayout timeLineLayout = new StackLayout()
            {
                Orientation = StackOrientation.Horizontal,
                Children =
                {
                    timeLabel,
                    minorLabel
                },
            };
            // Init the scaleLabel
            scaleLabel = new Label()
            {
                VerticalOptions = LayoutOptions.End,
                Text = "   Hour       Min         Sec",
                TextColor = new Color(255, 255, 255),
                FontSize = 20,
                IsVisible = true,
            };

            //Init the showTimeLayout
            StackLayout showTimeLayout = new StackLayout()
            {
                Orientation = StackOrientation.Vertical,
                Children =
                {
                    timeLineLayout,
                    scaleLabel,
                },
            };
            // Setting the ShowTimeLayout layout. 
            AbsoluteLayout.SetLayoutBounds(showTimeLayout, new Rectangle(0.20 * layoutWidth, 0.20 * layoutTopHeight, 0.8 * screenWidth, 0.218 * screenHeight));

            // Init the startButton
            startButton = new Image()
            {
                HorizontalOptions = LayoutOptions.FillAndExpand,
                Source = "button1.png",
            };

            // Setting the startButton layout.
            AbsoluteLayout.SetLayoutBounds(startButton, new Rectangle(0.2 * layoutWidth, 0.65 * layoutTopHeight, 0.167*screenWidth, 0.167 * screenWidth));

            // Init the labButton
            labButton = new Image()
            {
                HorizontalOptions = LayoutOptions.FillAndExpand,
                Source = "button4.png",
            };
            // Setting the labButton layout.
            AbsoluteLayout.SetLayoutBounds(labButton, new Rectangle(0.60 * layoutWidth, 0.65 * layoutTopHeight, 0.167 * screenWidth, 0.167 * screenWidth));
            //Background
            BoxView backGroundBox = new BoxView()
            {
                Color = new Color(0, 0, 0),
                HeightRequest = layoutTopHeight,
                WidthRequest = screenWidth,
            };

            //Use ObservableCollection to contain item data.
            //If data changed, it will sync update to ListView.
            lapList = new ObservableCollection<LapListItem>();

            //Create ListView which contain Lap items
            lapListView = new ListView()
            {
                RowHeight = (int)(0.1667 * screenWidth),
                ItemTemplate = new DataTemplate(() =>
                {
                    Label index = new Label()
                    {
                        TextColor = Color.Black,
                        FontSize = 25,
                    };
                    //Bind label's text and itemdata's Index property
                    index.SetBinding(Label.TextProperty, new Binding("Index", BindingMode.OneWay, null, null, "   Lap {0:d}"));
                    Label time = new Label()
                    {
                        HorizontalTextAlignment = TextAlignment.End,
                        TextColor = Color.Black,
                        FontSize = 25,
                    };
                    // Bind label's text and itemdata's Time property
                    time.SetBinding(Label.TextProperty, new Binding("Time", BindingMode.OneWay, null, null, "    |    {0:d}"));
                    Label minor = new Label()
                    {
                        VerticalOptions = LayoutOptions.StartAndExpand,
                        TextColor = Color.Black,
                        FontSize = 25,
                    };
                    // Bind label's text and itemdata's Minor property
                    minor.SetBinding(Label.TextProperty, new Binding("Minor", BindingMode.OneWay, null, null, ".{0:d}"));
                    return new ViewCell
                    {
                        View = new StackLayout
                        {
                            Orientation = StackOrientation.Horizontal,
                            Children =
                            {
                                index, time, minor
                            }
                        }
                    };
                }),
                ItemsSource = lapList,
            };

            this.Content = new StackLayout()
            {
                Spacing = 1,
                Children =
                {
                    //First layout contain time display UI
                    new AbsoluteLayout()
                    {
                        Children =
                        {
                            backGroundBox,
                            showTimeLayout,
                            startButton,
                            labButton,
                        }
                    },
                    //Second layout contain the ListView to display Lap time.
                    new StackLayout()
                    {
                        HorizontalOptions = LayoutOptions.FillAndExpand,
                        VerticalOptions = LayoutOptions.FillAndExpand,
                        Orientation = StackOrientation.Horizontal,
                        IsVisible = true,
                        Children =
                        {
                            lapListView
                        }
                    }
                }
            };
        }

        /// <summary>
        /// item data for ListView item.
        /// </summary>
        public class LapListItem
        {
            public int Index { get; set; }
            public string Time { get; set; }
            public string Minor { get; set; }
            public LapListItem(int index, string time, string minor)
            {
                Index = index;
                Time = time;
                Minor = minor;
            }
        }
    }
}
